# -*- coding: utf-8 -*-

"""
Provides hypothesis strategies for `ROS geometry_msgs`_.

.. _ROS geometry_msgs:
   http://wiki.ros.org/geometry_msgs

"""

from collections import namedtuple
from hypothesis.strategies import composite

from hypothesis_ros.messages.std_msgs import header, _Header
from hypothesis_ros.message_fields import (
    array,
    float64,
    string
)

_Point = namedtuple('Point', 'x y z')
_Quaternion = namedtuple('Quaternion', 'x y z w')
_Pose = namedtuple('Pose', 'position orientation')
_PoseWithCovariance = namedtuple('PoseWithCovariance', 'pose covariance')
_Transform = namedtuple('Transform', 'translation rotation')
_TransformStamed = namedtuple('TransformStamed', 'header child_frame_id transform')
_Vector3 = namedtuple('Vector3', 'x y z')

@composite
def point(draw, x=float64(), y=float64(), z=float64()):
    """
    Generate value for ROS geometry message type "point".

    Parameters
    ----------
    x : hypothesis.strategies.floats()
        Strategy to generate x value. (Default: Default hypothesis strategy.)
    y : hypothesis.strategies.floats()
        Strategy to generate y value. (Default: Default hypothesis strategy.)
    z : hypothesis.strategies.floats()
        Strategy to generate z value. (Default: Default hypothesis strategy.)

    """
    x_value, y_value, z_value = draw(x), draw(y), draw(z)
    assert isinstance(x_value, float), 'drew invalid x={x_value} from {x} for float64 field'.format(x_value, x)
    assert isinstance(y_value, float), 'drew invalid y={y_value} from {y} for float64 field'.format(y_value, y)
    assert isinstance(z_value, float), 'drew invalid y={z_value} from {z} for float64 field'.format(z_value, z)
    return _Point(x_value, y_value, z_value)


@composite
def quaternion(draw, x=float64(), y=float64(), z=float64(), w=float64()):
    """
    Generate value for ROS geometry message type "quaternion".

    Parameters
    ----------
    x : hypothesis.strategies.floats()
        Strategy to generate x value. (Default: Default hypothesis strategy.)
    y : hypothesis.strategies.floats()
        Strategy to generate y value. (Default: Default hypothesis strategy.)
    z : hypothesis.strategies.floats()
        Strategy to generate z value. (Default: Default hypothesis strategy.)
    w : hypothesis.strategies.floats()
        Strategy to generate w value. (Default: Default hypothesis strategy.)

    """
    x_value, y_value, z_value, w_value = draw(x), draw(y), draw(z), draw(w)
    assert isinstance(x_value, float), 'drew invalid x={x_value} from {x} for float64 field'.format(x_value, x)
    assert isinstance(y_value, float), 'drew invalid y={y_value} from {y} for float64 field'.format(y_value, y)
    assert isinstance(z_value, float), 'drew invalid y={z_value} from {z} for float64 field'.format(z_value, z)
    assert isinstance(w_value, float), 'drew invalid y={w_value} from {w} for float64 field'.format(w_value, w)
    return _Quaternion(x_value, y_value, z_value, w_value)


@composite
def pose(draw, position=point(), orientation=quaternion()):
    """
    Generate value for ROS geometry message type "pose".

    Parameters
    ----------
    position : hypothesis_ros.ros1.point()
        Strategy to generate position value. (Default: Default hypothesis_ros strategy.)
    orientation : hypothesis_ros.ros1.quaternion()
        Strategy to generate orientation value. (Default: Default hypothesis-ros strategy.)

    """
    position_value, orientation_value = draw(position), draw(orientation)
    assert isinstance(position_value, _Point), 'drew invalid position={position_value} from {position} for _Point field'.format(position_value, position)
    assert isinstance(orientation_value, _Quaternion), 'drew invalid orientation={orientation_value} from {orientation} for _Quaternion field'.format(orientation_value, orientation)
    return _Pose(position_value, orientation_value)


@composite
def pose_with_covariance(draw, pose=pose(), covariance=array(elements=float64(), min_size=36, max_size=36)):
    """
    Generate value for ROS geometry message type "PoseWithCovariance".
    """
    pose_value, covariance_value= draw(pose), draw(covariance)
    # TODO: add validation for covariance_value
    return _PoseWithCovariance(pose_value, covariance_value)


@composite
def vector3(draw, x=float64(), y=float64(), z=float64()):
    """
    Generate value for ROS geometry message type "vector3".

    Parameters
    ----------
    x : hypothesis.strategies.floats()
        Strategy to generate x value. (Default: Default hypothesis strategy.)
    y : hypothesis.strategies.floats()
        Strategy to generate y value. (Default: Default hypothesis strategy.)
    z : hypothesis.strategies.floats()
        Strategy to generate z value. (Default: Default hypothesis strategy.)

    """
    x_value, y_value, z_value = draw(x), draw(y), draw(z)
    assert isinstance(x_value, float), 'drew invalid x={x_value} from {x} for float64 field'.format(x_value, x)
    assert isinstance(y_value, float), 'drew invalid y={y_value} from {y} for float64 field'.format(y_value, y)
    assert isinstance(z_value, float), 'drew invalid y={z_value} from {z} for float64 field'.format(z_value, z)
    return _Vector3(x_value, y_value, z_value)


@composite
def transform(draw, translation=vector3(), rotation=quaternion()):
    """
    Generate value for ROS geometry message type "transform".

    Parameters
    ----------
    position : hypothesis_ros.ros1.vector3()
        Strategy to generate translation value. (Default: Default hypothesis_ros strategy.)
    orientation : hypothesis_ros.ros1.quaternion()
        Strategy to generate rotation value. (Default: Default hypothesis-ros strategy.)

    """
    translation_value, rotation_value = draw(translation), draw(rotation)
    assert isinstance(translation_value, _Vector3), 'drew invalid translation={translation_value} from {translation} for _Vector3 field'.format(translation_value, translation)
    assert isinstance(rotation_value, _Quaternion), 'drew invalid rotation={rotation_value} from {rotation} for _Quaternion field'.format(rotation_value, rotation)
    return _Transform(translation_value, rotation_value)


@composite
def transform_stamped(draw, header=header(), child_frame_id=string(), transform=transform()):
    """
    Generate value for ROS geometry message type "TransformStamped".

    Parameters
    ----------
    header : hypothesis_ros.messages.std_msgs.header()
        Strategy to generate header value. (Default: Default hypothesis-ros strategy.)
    child_frame_id : hypothesis_ros.message_fields.string()
        Strategy to generate child_frame_id value. (Default: Default hypothesis-ros strategy.)
    transform : hypothesis_ros.messages.geometry_msgs.transform()
        Strategy to generate transform value. (Default: Default hypothesis-ros strategy.)

    """
    header_value = draw(header)
    child_frame_id_value = draw(child_frame_id)
    transform_value = draw(transform)
    assert isinstance(header_value, _Header), 'drew invalid header={header_value} from {header} for _Header field'.format(header_value, header)
    assert isinstance(child_frame_id_value, str), 'drew invalid child_frame_id={child_frame_id_value} from {child_frame_id} for string field'.format(child_frame_id_value, child_frame_id)
    assert isinstance(transform_value, _Transform),  'drew invalid transform={transform_value} from {transform} for _Transform field'.format(transform_value, transform)
    return _TransformStamed(header_value, child_frame_id_value, transform_value)



# -*- coding: utf-8 -*-

"""
Provides hypothesis strategies for `ROS sensor_msgs`_.

.. _ROS sensor_msgs:
   http://wiki.ros.org/sensor_msgs

"""

from collections import namedtuple
from hypothesis.strategies import composite, sampled_from

from hypothesis_ros.messages.geometry_msgs import vector3, quaternion
from hypothesis_ros.messages.std_msgs import header
from hypothesis_ros.message_fields import (  # pylint: disable=redefined-builtin
    array,
    bool,
    float32,
    float64,
    time,
    uint8,
    uint32,
)

_RegionOfInterest = namedtuple('RegionOfInterest', 'x_offset y_offset height width do_rectify')
_Imu = namedtuple('Imu', 'header orientation orientation_covariance angular_velocity angular_velocity_covariance linear_acceleration linear_acceleration_covariance')
_CompressedImage = namedtuple('CompressedImage', 'header format data')
_Image = namedtuple('Image', 'header height width encoding step is_bigendian data')


IMAGE_ENCODINGS = ["rgb8", "rgba8", "rgb16", "rgba16", "bgr8", "bgra8", "bgr16", "bgra16",
                   "mono8", "mono16",
                   "8UC1", "8UC2", "8UC3", "8UC4", "8SC1", "8SC2", "8SC3", "8SC4",
                   "16UC1", "16UC2", "16UC3", "16UC4", "16SC1", "16SC2", "16SC3", "16SC4",
                   "32SC1", "32SC2", "32SC3", "32SC4", "32FC1", "32FC2", "32FC3", "32FC4",
                   "64FC1", "64FC2", "64FC3", "64FC4",
                   "bayer_rggb8", "bayer_bggr8", "bayer_gbrg8", "bayer_grbg8",
                   "bayer_rggb16", "bayer_bggr16", "bayer_gbrg16", "bayer_grbg16",
                   "yuv422"]
"""Image message encodings."""

@composite
def region_of_interest(draw,  # pylint: disable=too-many-arguments
                       x_offset=uint32(),
                       y_offset=uint32(),
                       height=uint32(),
                       width=uint32(),
                       do_rectify=bool()):  # pylint: disable=no-value-for-parameter
    """
    Generate values for ROS sensor_msgs/RegionOfInterest.msg.

    Parameters
    ----------
    x_offset : hypothesis_ros.message_fields.uint32()
        Strategy to generate x_offset value. (Default: Default hypothesis_ros strategy.)
    y_offset : hypothesis_ros.message_fields.uint32()
        Strategy to generate y_offset value. (Default: Default hypothesis_ros strategy.)
    height : hypothesis_ros.message_fields.uint32()
        Strategy to generate height value. (Default: Default hypothesis_ros strategy.)
    width : hypothesis_ros.message_fields.uint32()
        Strategy to generate width value. (Default: Default hypothesis_ros strategy.)
    do_rectify : hypothesis_ros.message_fields.bool()
        Strategy to generate do_rectify value. (Default: Default hypothesis_ros strategy.)

    """
    x_offset_value = draw(x_offset)
    y_offset_value = draw(y_offset)
    height_value = draw(height)
    width_value = draw(width)
    do_rectify_value = draw(do_rectify)
    assert isinstance(x_offset_value, int), 'drew invalid x_offset={x_offset_value} from {x_offset} for uint32 field'.format(x_offset_value, x_offset)
    assert isinstance(y_offset_value, int), 'drew invalid y_offset={y_offset_value} from {y_offset} for uint32 field'.format(y_offset_value, y_offset)
    assert isinstance(height_value, int), 'drew invalid height={height_value} from {height} for uint32 field'.format(height_value, height)
    assert isinstance(width_value, int), 'drew invalid width={width_value} from {width} for uint32 field'.format(width_value, width)
    assert isinstance(do_rectify_value, int), 'drew invalid do_rectify={do_rectify_value} from {do_rectify} for bool field'.format(do_rectify_value, do_rectify)  # bool is subclass of int
    return _RegionOfInterest(x_offset_value, y_offset_value, height_value, width_value, do_rectify_value)


@composite
def imu(draw,
        header=header(),
        orientation=quaternion(),
        orientation_covariance=array(elements=float64(), min_size=9, max_size=9),
        angular_velocity=vector3(),
        angular_velocity_covariance=array(elements=float64(), min_size=9, max_size=9),
        linear_acceleration=vector3(),
        linear_acceleration_covariance=array(elements=float64(), min_size=9, max_size=9)
       ):
    """
    Generate values for ROS sensor_msgs/Imu.msg.

    Parameters
    ----------
    header : hypothesis_ros.std_msgs.header()
        Strategy to generate header value. (Default: Default hypothesis_ros strategy.)
    orientation : hypothesis_ros.geometry_msgs.Quaternion()
        Strategy to generate orientation value. (Default: Default hypothesis_ros strategy.)
    orientation_covariance : hypothesis_ros.message_fields.array()
        Strategy to generate orientation_covariance value. (Default: Customized to 9 elements of type float64().)
    angular_velocity : hypothesis_ros.messages.geometry_msgs.vector3()
        Strategy to generate angular_velocity value. (Default: Default hypothesis_ros strategy.)
    angular_velocity_covariance : hypothesis_ros.message_fields.array()
        Strategy to generate angular_velocity_covariance value. (Default: Customized to 9 elements of type float64().)
    linear_acceleration : hypothesis_ros.messages.geometry_msgs.vector3()
        Strategy to generate linear_acceleration value. (Default: Default hypothesis_ros strategy.)
    linear_acceleration_covariance : hypothesis_ros.messages.message_fields.array()
        Strategy to generate linear_acceleration_covariance value. (Default: Customized to 9 elements of type float64().)

    """
    header_value = draw(header)
    orientation_value = draw(orientation)
    orientation_covariance_value = draw(orientation_covariance)
    angular_velocity_value = draw(angular_velocity)
    angular_velocity_covariance_value = draw(angular_velocity_covariance)
    linear_acceleration_value = draw(linear_acceleration)
    linear_acceleration_covariance_value = draw(linear_acceleration_covariance)
    # TODO: add validation
    return _Imu(header_value,
                orientation_value,
                orientation_covariance_value,
                angular_velocity_value,
                angular_velocity_covariance_value,
                linear_acceleration_value,
                linear_acceleration_covariance_value
               )


@composite
def compressed_image(draw, header=header(), format=sampled_from(['jpg', 'png']), data=array(elements=uint8())):
    """
    Generate values for ROS sensor_msgs/CompressedImage.msg.

    Parameters
    ----------
    header : hypothesis_ros.std_msgs.header()
        Strategy to generate header value. (Default: Default hypothesis_ros strategy.)
    format : hypothesis.strategies.sampled_from()
        Strategy to generate format value. (Default: Customized hypothesis strategy.)
    data : hypothesis_ros.message_fields.array()
        Strategy to generate format value. (Default: Customized to elements of type uint8().)

    """
    header_value = draw(header)
    format_value = draw(format)
    data_value = draw(data)
    return _CompressedImage(header_value,
                            format_value,
                            data_value
                           )


@composite
def image(draw,
          header=header(),
          height=uint32(),
          width=uint32(),
          encoding=sampled_from(IMAGE_ENCODINGS),
          step=uint32(),
          is_bigendian=uint8(),
          data=array(elements=uint8(), max_size=10000**10000)):
    """
    Generate values for ROS sensor_msgs/Image.msg.

    Be aware that the element count of the "data" field value is not generated dependent on
    steps and rows right now. Configuration of field "data" element size requires attention
    to avoid exceptions raised in underlying hypothesis functionality.

    Parameters
    ----------
    header : hypothesis_ros.messages.std_msgs.header()
        Strategy to generate header values. (Default: Default hypothesis_ros strategy.)
    height : hypothesis_ros.message_fields.uint32()
        Strategy to generate height value. (Default: Default hypothesis_ros strategy.)
    width : hypothesis_ros.message_fields.uint32()
        Strategy to generate width value. (Default: Default hypothesis_ros strategy.)
    encoding : hypothesis.sampled_from()
        Strategy to generate encoding value. For possible values refer to
        include/sensor_msgs/image_encodings.h . (Default: hypothesis strategy with reasonable configuration.)
    is_bigendian : hypothesis_ros.message_fields.uint8()
        Strategy to generate bigendian value. (Default: Default hypothesis_ros strategy.)
    data : hypothesis_ros.message_fields.array(elements=uint8())
        Strategy to generate matrix data values. Size is steps x rows. (Default: hypothesis_ros strategy with reasonable configuration.)

    """
    header_value = draw(header)
    height_value = draw(height)
    width_value = draw(width)
    encoding_value = draw(encoding)
    step_value = draw(step)
    is_bigendian_value = draw(is_bigendian)
    data_value = draw(data)
    return _Image(header_value, height_value, width_value, encoding_value, step_value, is_bigendian_value, data_value)



# -*- coding: utf-8 -*-

"""
Provides hypothesis strategies for `ROS std_msgs`_.

.. _ROS std_msgs:
   http://wiki.ros.org/std_msgs

"""

from collections import namedtuple
from hypothesis.strategies import composite

from hypothesis_ros.message_fields import (
    uint32,
    string,
    time
)

_Header = namedtuple('Header', 'seq stamp frame_id')

@composite
def header(draw, seq=uint32(), stamp=time(), frame_id=string()):
    """
    Generate value for ROS standard message type "header".

    Parameters
    ----------
    seq : hypothesis_ros.message_fields.uint32()
        Strategy to generate seq value. (Default: Default hypothesis_ros strategy.)
    stamp : hypothesis_ros.message_fields.time()
        Strategy to generate stamp value. (Default: Default hypothesis_ros strategy.)
    frame_id : hypothesis_ros.message_fields.string()
        Strategy to generate frame_id value. (Default: Default hypothesis_ros strategy.)

    """
    seq_value = draw(seq)
    stamp_value = draw(stamp)
    frame_id_value = draw(frame_id)
    assert isinstance(seq_value, int), 'drew invalid seq={seq_value} from {seq} for uint32 field'.format(seq_value, seq)
    assert isinstance(stamp_value.secs, int), 'drew invalid stamp.secs={stamp_value} from {stamp} for int field'.format(stamp_value.secs, stamp)
    assert isinstance(stamp_value.nsecs, int), 'drew invalid stamp.nsecs={stamp_value} from {stamp} for int field'.format(stamp_value.nsecs, stamp)
    assert isinstance(frame_id_value, str), 'drew invalid frame_id={frame_id_value} from {frame_id} for str field'.format(frame_id_value, frame_id)
    return _Header(seq_value, stamp_value, frame_id_value)



# -*- coding: utf-8 -*-

"""
Provides hypothesis strategies for `ROS parameter types`_.

.. _ROS parameter types:
   http://wiki.ros.org/Parameter%20Server#Parameter_Types

"""

import datetime
from hypothesis.strategies import (
    datetimes,
    defines_strategy,
    lists,
    floats
)
from hypothesis_ros.message_fields import int32, string, bool, array as list

__all__ = ['int32', 'string', 'bool', 'list']
"""Make strategies from hypothesis_ros.message_fields available in this module."""

DATE_MIN_VALUE = datetime.datetime(datetime.MINYEAR, 1, 1, 0, 0)
"""date: Minimal ISO8601 Date value (0001-01-01 00:00:00)."""
DATE_MAX_VALUE = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59)
"""date: Maximal ISO8601 Date value (9999-12-31 23:59:00)."""

DOUBLE_MIN_VALUE = -3.4028235e+38
"""float: Minimal Double value."""
DOUBLE_MAX_VALUE = +3.4028235e+38
"""float: Maximal Double value."""


@defines_strategy
def date(min_value=DATE_MIN_VALUE, max_value=DATE_MAX_VALUE):
    """
    Generate value for ROS parameter type "date".

    Parameters
    ----------
    min_value : datetime.datetime
        Minimal value to generate.
    max_value : datetime.datetime
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.datetimes()
        Strategy with values taken from datetime library.

    """
    return datetimes(min_value, max_value)


@defines_strategy
def double(min_value=DOUBLE_MIN_VALUE, max_value=DOUBLE_MAX_VALUE,
            allow_nan=False, allow_infinity=False):
    """
    Generate value for ROS paramter type "double".

    Parameters
    ----------
    min_value : float
        Minimal value to generate.
    max_value : float
        Maximal value to generate.
    allow_nan : boolean
        Generate "not a number"? (Default: false)
    allow_infinity : boolean
        Generate "infinity"? (Default: false)

    Returns
    -------
    hypothesis.strategies.floats()
        Strategy with preconfigured default values.

    """
    return floats(min_value, max_value, allow_nan, allow_infinity)



# -*- coding: utf-8 -*-

"""
Provides hypothesis strategies for `ROS tf2_msgs`_.

.. _ROS tf2_msgs:
   http://wiki.ros.org/tf2_msgs

"""


from collections import namedtuple
from hypothesis.strategies import composite

from hypothesis_ros.messages.geometry_msgs import transform_stamped
from hypothesis_ros.message_fields import array


_TFMessage = namedtuple('TFMessage', 'transforms')


@composite
def tfmessage(draw, transforms=array(elements=transform_stamped())):
    """
    Generate value for ROS TFMessage.

    Parameters
    ----------
    transforms : hypothesis_ros.message_field.array()
        Strategy to generate transforms value. (Default: Default hypothesis_ros strategy
        with elements of type hypothesis_ros.message.geometry_msgs.transform_stamped().)

    """
    transforms_value = draw(transforms_value)
    return _TFMessage(transforms_value)



# -*- coding: utf-8 -*-

"""
Provides hypothesis strategies for `ROS message fields`_.

.. _ROS message fields:
   http://wiki.ros.org/msg#Fields

"""

from collections import namedtuple
import datetime
from hypothesis.errors import InvalidArgument
from hypothesis.strategies import (
    binary,
    booleans,
    composite,
    defines_strategy,
    floats,
    integers,
    lists,
)

INT8_MIN_VALUE = -1 * (2 ** 7)
"""int: Minimal Int8 value."""
INT8_MAX_VALUE = (2 ** 7) - 1
"""int: Maximal Int8 value."""

UINT8_MIN_VALUE = 0
"""int: Minimal UInt8 value."""
UINT8_MAX_VALUE = (2 ** 8) - 1
"""int: Maximal UInt8 value."""

INT16_MIN_VALUE = -1 * (2 ** 15)
"""int: Minimal Int16 value."""
INT16_MAX_VALUE = (2 ** 15) - 1
"""int: Maximal Int16 value."""

UINT16_MIN_VALUE = 0
"""int: Minimal UInt16 value."""
UINT16_MAX_VALUE = (2 ** 16) - 1
"""int: Maximal UInt16 value."""

INT32_MIN_VALUE = -1 * (2 ** 31)
"""int: Minimal Int32 value."""
INT32_MAX_VALUE = (2 ** 31) - 1
"""int: Maximal Int32 value."""

UINT32_MIN_VALUE = 0
"""int: Minimal UInt32 value."""
UINT32_MAX_VALUE = (2 ** 32) - 1
"""int: Maximal UInt32 value."""

INT64_MIN_VALUE = -1 * (2 ** 63)
"""long: Minimal Int64 value."""
INT64_MAX_VALUE = (2 ** 63) - 1
"""long: Maximal Int64 value."""

UINT64_MIN_VALUE = 0
"""long: Minimal UInt64 value."""
UINT64_MAX_VALUE = (2 ** 64) - 1
"""long: Maximal UInt64 value."""

FLOAT32_MIN_VALUE = -3.4e+38
"""float: Minimal Float32 value."""
FLOAT32_MAX_VALUE = +3.4e+38
"""float: Maximal Float32 value."""

FLOAT64_MIN_VALUE = -1.7E+308
"""float: Minimal Float32 value."""
FLOAT64_MAX_VALUE = +1.7E+308
"""float: Maximal Float32 value."""

STRING_MIN_SIZE = 0
"""int: Minimal string size."""
STRING_MAX_SIZE = 1000
"""int: Maximal string size."""

_Time = namedtuple('Time', 'secs nsecs')
_Duration = namedtuple('Duration', 'secs nsecs')


@defines_strategy
def bool():  # pylint: disable=redefined-builtin
    """
    Generate value for ROS builtin message type "bool".

    Returns
    -------
    hypothesis.strategies.booleans()
        Strategy with preconfigured default values.

    """
    return booleans()


@defines_strategy
def int8(min_value=INT8_MIN_VALUE, max_value=INT8_MAX_VALUE):
    """
    Generate value for ROS builtin message type "int8".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (INT8_MIN_VALUE <= min_value <= max_value <= INT8_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def uint8(min_value=UINT8_MIN_VALUE, max_value=UINT8_MAX_VALUE):
    """
    Generate value for ROS builtin message type "uint8".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (UINT8_MIN_VALUE <= min_value <= max_value <= UINT8_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def int16(min_value=INT16_MIN_VALUE, max_value=INT16_MAX_VALUE):
    """
    Generate value for ROS builtin message type "int16".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (INT16_MIN_VALUE <= min_value <= max_value <= INT16_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def uint16(min_value=UINT16_MIN_VALUE, max_value=UINT16_MAX_VALUE):
    """
    Generate value for ROS builtin message type "uint16".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (UINT16_MIN_VALUE <= min_value <= max_value <= UINT16_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def int32(min_value=INT32_MIN_VALUE, max_value=INT32_MAX_VALUE):
    """
    Generate value for ROS builtin message type "int32".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (INT32_MIN_VALUE <= min_value <= max_value <= INT32_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def uint32(min_value=UINT32_MIN_VALUE, max_value=UINT32_MAX_VALUE):
    """
    Generate value for ROS builtin message type "uint32".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (UINT32_MIN_VALUE <= min_value <= max_value <= UINT32_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def int64(min_value=INT64_MIN_VALUE, max_value=INT64_MAX_VALUE):
    """
    Generate value for ROS builtin message type "int64".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (INT64_MIN_VALUE <= min_value <= max_value <= INT64_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def uint64(min_value=UINT64_MIN_VALUE, max_value=UINT64_MAX_VALUE):
    """
    Generate value for ROS builtin message type "uint64".

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.

    Returns
    -------
    hypothesis.strategies.integers()
        Strategy with preconfigured default values.

    """
    if not (UINT64_MIN_VALUE <= min_value <= max_value <= UINT64_MAX_VALUE): raise InvalidArgument
    return integers(min_value, max_value)


@defines_strategy
def float32(min_value=FLOAT32_MIN_VALUE, max_value=FLOAT32_MAX_VALUE,
            allow_nan=False, allow_infinity=False):
    """
    Generate value for ROS builtin message type "float32".

    TODO: Map Hypothesis values to YAML [.inf, -.Inf, .NAN].
          http://www.yaml.org/refcard.html

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.
    allow_nan : boolean
        Generate "not a number"? (Default: false)
    allow_infinity : boolean
        Generate "not a number"? (Default: false)

    Returns
    -------
    hypothesis.strategies.floats()
        Strategy with preconfigured default values.

    """
    # TODO
    return floats(min_value, max_value, allow_nan, allow_infinity)


@defines_strategy
def float64(min_value=FLOAT64_MIN_VALUE, max_value=FLOAT64_MAX_VALUE,
            allow_nan=False, allow_infinity=False):
    """
    Generate value for ROS builtin message type "float64".

    TODO: Map Hypothesis values to YAML [.inf, -.Inf, .NAN].
          http://www.yaml.org/refcard.html

    Parameters
    ----------
    min_value : int
        Minimal value to generate.
    max_value : int
        Maximal value to generate.
    allow_nan : boolean
        Generate "not a number"? (Default: false)
    allow_infinity : boolean
        Generate "not a number"? (Default: false)

    Returns
    -------
    hypothesis.strategies.floats()
        Strategy with preconfigured default values.

    """
    # TODO
    return floats(min_value, max_value, allow_nan, allow_infinity)


@defines_strategy
def string(min_size=STRING_MIN_SIZE, max_size=STRING_MAX_SIZE):
    """
    Generate value for ROS builtin message type "string".

    Parameters
    ----------
    min_size : int
        Minimal size to generate.
    max_size : int
        Maximal size to generate.

    Returns
    -------
    hypothesis.strategies.binary()
        Strategy with preconfigured default values.

    """
    if not (STRING_MIN_SIZE <= min_size <= max_size <= STRING_MAX_SIZE): raise InvalidArgument
    # average_size parameter is deprecated
    return binary(min_size=min_size, max_size=max_size)


@composite
def time(draw, secs=uint32(), nsecs=uint32()):
    """
    Generate value for ROS builtin message type "time".

    Parameters
    ----------
    secs: hypothesis_ros.message_fields.uint32()
        Seconds.
    nsecs: hypothesis_ros.message_fields.uint32()
        Nano seconds.

    Returns
    -------
    _Time()
        Strategy with preconfigured default values.

    """
    secs_value, nsecs_value = draw(secs), draw(nsecs)
    assert isinstance(secs_value, int), 'drew invalid sec={secs_value} from {secs} for integer field'.format(secs_value, secs)
    assert isinstance(nsecs_value, int), 'drew invalid nsec={nsecs_value} from {nsecs} for integer field'.format(nsecs_value, nsecs)
    return _Time(secs_value, nsecs_value)


@composite
def duration(draw, secs=int32(), nsecs=int32()):
    """
    Generate value for ROS builtin message type "duration".

    Parameters
    ----------
    secs: hypothesis_ros.message_fields.int32()
        Seconds.
    nsecs: hypothesis_ros.message_fields.int32()
        Nano seconds.

    Returns
    -------
    _Duration()
        Namedtuple with drawn values for secs and nsecs.

    """
    secs_value, nsecs_value = draw(secs), draw(nsecs)
    assert isinstance(secs_value, int), 'drew invalid sec={secs_value} from {secs} for integer field'.format(secs_value, secs)
    assert isinstance(nsecs_value, int), 'drew invalid nsec={nsecs_value} from {nsecs} for integer field'.format(nsecs_value, nsecs)
    return _Duration(secs_value, nsecs_value)


@defines_strategy
def array(elements=None, min_size=None, max_size=None, unique_by=None, unique=None):
    """
    Generate variable length array with ROS builtin message types as elements.
    To generate a fixed length array define `min_size == max_size`.

    TODO: Assert that strategy for elements is from supported strategies.

    Parameters
    ----------
    elements: hypothesis_ros.message_fields
        Strategies for pritive types from hypothesis_ros.message_fields.
    min_size: integer
        Minimal size of the array.
    max_size: integer
        Maximal size of the array.
    unique_by: callable
        Function returning a hashable type when given a value from elements.
        The resulting array (list) will satisfy `unique_by(result[i]) != unique_by(result[j])`.
    unique: boolean
        Function returning a hashable type. For comparison of directy object equality.

    Returns
    -------
    list
        A variable or fixed length array containing values drawn from elements with
        length in the interval [min_size, max_size] (no bounds in that direction
        if these are None).

    """
    if not (min_size <= max_size): raise InvalidArgument
    return lists(elements=elements, min_size=min_size, max_size=max_size, unique_by=unique_by, unique=unique)


# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('requirements.txt') as f:
    install_requires = [line.strip() for line in f.readlines()]

with open('test-requirements.txt') as f:
    test_requires = [line.strip() for line in f.readlines()]

setup(
    name="hypothesis-ros",
    version="0.2.0",
    url="https://github.com/ros-testing/hypothesis-ros",
    license='Apache License 2.0',
    author="Florian Kromer",
    author_email="florian.kromer@mailbox.org",
    description="Property Based Testing for the ROS node level.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=install_requires,
    test_requires=test_requires,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development :: Testing'
    ],
)



# -*- coding: utf-8 -*-

"""
Provides hypothesis strategies for `ROS stereo_msgs`_.

.. _ROS stereo_msgs:
   http://wiki.ros.org/stereo_msgs

"""

from collections import namedtuple
from hypothesis.strategies import composite

from hypothesis_ros.messages.geometry_msgs import transform_stamped
from hypothesis_ros.messages.sensor_msgs import image, region_of_interest
from hypothesis_ros.messages.std_msgs import header
from hypothesis_ros.message_fields import (
    array,
    float32
)


_DisparityImage = namedtuple('DisparityImage', 'header image f T valid_window min_disparity max_disparity delta_d')


@composite
def disparity_image(draw, header=header(), image=image(), f=float32(), t=float32(), valid_window=region_of_interest(), min_disparity=float32(), max_disparity=float32(), delta_d=float32()):
    """
    Generate value for ROS DisparityImage.msg.

    Parameters
    ----------
    header : hypothesis_ros.messages.std_msgs.header()
        Strategy to generate header value. (Default: Default hypothesis-ros strategy.)
    image : hypothesis_ros.messages.sensor_msgs.image()
        Strategy to generate image value. (Default: Default hypothesis-ros strategy.)
    f : hypothesis_ros.message_field.float32()
        Strategy to generate f value. (Default: Default hypothesis-ros strategy.)
    t : hypothesis_ros.message_field.float32()
        Strategy to generate T value. (Default: Default hypothesis-ros strategy.)
    valid_window : hypothesis_ros.messages.sensor_msgs.region_of_interest()
    min_disparity : hypothesis_ros.message_field.float32()
        Strategy to generate T value. (Default: Default hypothesis-ros strategy.)
    max_disparity : hypothesis_ros.message_field.float32()
        Strategy to generate T value. (Default: Default hypothesis-ros strategy.)
    delta_d : hypothesis_ros.message_field.float32()
        Strategy to generate T value. (Default: Default hypothesis-ros strategy.)

    """
    header_value = draw(header)
    image_value = draw(image)
    f_value = draw(f)
    t_value = draw(t)
    valid_window_value = draw(valid_window)
    min_disparity_value = draw(min_disparity)
    max_disparity_value = draw(max_disparity)
    delta_d_value = draw(delta_d)
    return _DisparityImage(header_value,
                           image_value,
                           f_value,
                           t_value,
                           valid_window_value,
                           min_disparity_value,
                           max_disparity_value,
                           delta_d_value)
